# -*- coding: utf-8 -*-
from blindspin import spinner

import click

import emoji

from .. import api
from .. import cli
from .. import options


@cli.cli.group()
def config():
    """Update the configuration for your app."""
    pass


@config.command(name='list')
@options.app()
def list_command(app):
    """List environment variables."""

    cli.user()
    click.echo('Fetching config… ', nl=False)

    with spinner():
        config = api.Config.get(app)

    click.echo(
        click.style('\b' + emoji.emojize(':heavy_check_mark:'), fg='green'))

    if config:
        click.echo(click.style('Storyscript variables:', dim=True))
        for name, value in config.items():
            if not isinstance(value, dict):
                click.echo(click.style(name, fg='green') + f':  {value}')

        click.echo('')
        click.echo(click.style('Service variables:', dim=True))
        for name, value in config.items():
            if isinstance(value, dict):
                click.echo(click.style(name, bold=True))
                for _name, _value in value.items():
                    click.echo(
                        '  ' + click.style(_name, fg='green') + f':  {_value}')

    else:
        click.echo(click.style('No configuration set yet.', bold=True))
        click.echo(
            '\nSet Storyscript environment '
            + click.style('$ ', dim=True)
            + click.style('story config set key=value', fg='magenta')
        )
        click.echo(
            'Set service environment '
            + click.style('$ ', dim=True)
            + click.style('story config set service.key=value', fg='magenta')
        )


@config.command(name='set')
@click.argument('variables', nargs=-1)
@click.option(
    '--message',
    '-m',
    nargs=1,
    default=None,
    help='(optional) Message why variable(s) were created.',
)
@options.app()
def set_command(variables, app, message):
    """
    Set one or more environment variables.

        $ story config set key=value foo=bar

    To set an environment variable for a specific service use

        $ story config set twitter.oauth_token=value

    """
    cli.user()

    click.echo('Fetching config… ', nl=False)
    with spinner():
        config = api.Config.get(app=app)
    click.echo(
        click.style('\b' + emoji.emojize(':heavy_check_mark:'), fg='green'))

    if variables:
        for keyval in variables:
            try:
                key, val = tuple(keyval.split('=', 1))
            except ValueError:
                click.echo(
                    f'Config variables must be of the form name=value.'
                    f'\nGot unexpected pair "{keyval}"',
                    err=True,
                )
                click.echo(set_command.__doc__.strip())
                return
            # TODO validate against a regexp pattern
            if '.' in key:
                service, key = tuple(key.split('.', 1))
                config.setdefault(service.lower(), {})[key.upper()] = val
            else:
                config[key.upper()] = val

            click.echo()
            click.echo(f" {click.style(key.upper(), fg='green')}: {val}")

        click.echo('\nSetting config and deploying new release…  ', nl=False)
        with spinner():
            release = api.Config.set(config=config, app=app, message=message)
        click.echo(click.style('\b' + emoji.emojize(':heavy_check_mark:'),
                               fg='green'))
        click.echo(
            f'Deployed new release… '
            + click.style(f'v{release["id"]}', bold=True, fg='magenta')
        )

    else:
        click.echo(set_command.__doc__.strip())


@config.command()
@click.argument('variables', nargs=-1)
@options.app()
def get(variables, app):
    """Get one or more environment variables."""

    cli.user()

    if variables:
        click.echo(f'Fetching config for {app}… ', nl=False)

        with spinner():
            config = api.Config.get(app=app)

        click.echo(click.style(emoji.emojize(':heavy_check_mark:'),
                               fg='green'))

        for name in variables:
            if '.' in name:
                service, name = tuple(name.split('.', 1))
                value = config[service.lower()].get(name.upper(), None)
            else:
                if name in config:
                    # could be a service here
                    value = config[name]
                else:
                    value = config.get(name.upper(), None)

            if value:
                if isinstance(value, dict):
                    for name, value in value.items():
                        click.echo(
                            click.style(name.upper(),
                                        fg='green') + f':  {value}'
                        )
                else:
                    click.echo(
                        click.style(name.upper(), fg='green') + f':  {value}')
            else:
                click.echo(
                    click.style(f'No variable named "{name.upper()}".',
                                fg='red')
                )
    else:
        click.echo(get.__doc__.strip())


@config.command(name='del')
@click.argument('variables', nargs=-1)
@click.option(
    '--message',
    '-m',
    nargs=1,
    default=None,
    help='(optional) Message why variable(s) were deleted.',
)
@options.app()
def del_command(variables, app, message):
    """
    Delete one or more environment variables
    """
    cli.user()
    if variables:

        click.echo('Fetching config… ', nl=False)
        with spinner():
            config = api.Config.get(app=app)
        click.echo(
            click.style(emoji.emojize(':heavy_check_mark:'), fg='green'))

        for key in variables:
            removed = False
            if key in config:
                if type(config.pop(key)) is dict:
                    click.echo(
                        click.style('Removed service', fg='red') + f': {key}')
                else:
                    removed = True
            elif key.upper() in config:
                config.pop(key.upper())
                removed = True
            elif '.' in key:
                service, key = tuple(key.split('.', 1))
                if service in config and key.upper() in config[service]:
                    config[service].pop(key.upper())
                    removed = True

            if removed:
                click.echo(
                    click.style('Removed', fg='red') + f': {key.upper()}')

        click.echo('\nSetting config and deploying new release… ', nl=False)
        with spinner():
            release = api.Config.set(config=config, app=app, message=message)
        click.echo(click.style('\b' + emoji.emojize(':heavy_check_mark:'),
                               fg='green'))
        click.echo(
            f'Deployed new release… '
            + click.style(f'v{release["id"]}', bold=True, fg='magenta')
        )

    else:
        click.echo(del_command.__doc__.strip())



# -*- coding: utf-8 -*-
import pkgutil

import click

import emoji

from story import cli

CHOICES = [
    'http',
    'every',
    'function',
    'if',
    'loop',
    'twitter',
    'slack-bot',
    'subscribe',
    'every',
    'websocket',
    '-',
]


@cli.cli.command()
@click.argument('story', default='-', type=click.Choice(CHOICES))
@click.argument(
    'output_file', default=None, type=click.Path(exists=False), required=False
)
def write(story, output_file=None):
    """Pre–defined Storyscripts for your app!"""

    # Support '$ story write http -` usecase.`
    if output_file == '-':
        output_file = None

    if story == '-':
        click.echo(click.style('Please specify a template:', bold=True))
        click.echo(
            click.style('  http', fg='cyan') + '      - serverless http')
        click.echo(
            click.style('  function', fg='cyan') + '  - generic function')
        click.echo(
            click.style('  if', fg='cyan') + '        - example if/then')
        click.echo(
            click.style('  loop', fg='cyan') + '      - example for loop')
        click.echo(click.style('  twitter', fg='cyan') + '   - stream Tweets')
        click.echo('')

        click.echo(
            click.style('Coming Soon',
                        bold=True) + ' (under active development):'
        )
        click.echo(click.style('  slack-bot', fg='cyan') + ' - Slack bot')
        click.echo(
            click.style('  subscribe', fg='cyan') + ' - event subscriptions')
        click.echo(
            click.style('  every', fg='cyan') + '     - periodically run this')
        click.echo(
            click.style('  websocket', fg='cyan') + ' - websocket support')
        click.echo('')

        click.echo(
            '  Run $ '
            + click.style('story write :template_name: ', fg='magenta')
            + emoji.emojize(':backhand_index_pointing_left:')
        )
        click.echo('')

        click.echo(click.style('Learn more:', bold=True))
        click.echo(
            '  - Examples: '
            + click.style('https://github.com/topics/storyscript-example',
                          fg='cyan')
        )
        click.echo(
            '  - Services: ' + click.style('https://hub.storyscript.io/',
                                           fg='cyan')
        )
        click.echo('')

    else:

        # Grab the story, from packaging...
        data = pkgutil.get_data('story', f'stories/{story}.story')

        # If output_file was passed, assume it was an interfactive session.
        if output_file:
            # Write to the file...
            with open(output_file, 'wb') as f:
                f.write(data)

            cmd = f'cat {output_file}'
            cmd = click.style(cmd, fg='magenta')
            click.echo(f'$ {cmd}', err=True)

        click.echo(data)

        app_name = cli.get_app_name_from_yml()
        if app_name is None:
            app_name = 'Not created yet'

        cli.track('App Bootstrapped',
                  {'App name': app_name, 'Template used': story})



# -*- coding: utf-8 -*-

import click

from .. import cli


@cli.cli.command()
def login():
    """Login to Storyscript Cloud via GitHub."""
    cli.user()
    click.echo("You're logged in as " + click.style(cli.data['email'],
                                                    fg='cyan') + '.')



# -*- coding: utf-8 -*-
from click.testing import CliRunner

from pytest import fixture

from story.main import cli


@fixture
def runner():
    return CliRunner()


def test_choose_template(runner):
    res = runner.invoke(cli, ['write'])
    assert res.exit_code == 0
    assert 'Please specify a template' in res.output



# -*- coding: utf-8 -*-

import click

from .cli import run


try:
    from . import cli
    _app = cli.get_app_name_from_yml()
except:
    _app = None


def app(allow_option=True):
    return click.option(
        '--app', '-a',
        default=_app,
        help=f'(required) [default: {_app}] app to run command against',
        callback=lambda context, p, app: cli.assert_project(
            context.command.name, app, _app, allow_option
        )
    )



# -*- coding: utf-8 -*-
import os
import subprocess
import sys

from blindspin import spinner

import click

import emoji

from .. import api
from .. import awesome
from .. import cli
from .. import options
from ..helpers.datetime import parse_psql_date_str, reltime


def maintenance(enabled: bool) -> str:
    if enabled:
        return 'in maintenance'
    else:
        return 'running'


@cli.cli.group()
def apps():
    """Create, list, and manage apps on Storyscript Cloud."""
    pass


@apps.command(name='list')
def list_command():
    """List apps that you have access to."""
    from texttable import Texttable

    cli.user()

    with spinner():
        res = api.Apps.list()

    count = 0
    # Heads up! Texttable does not like colours.
    # So don't use click.style here.
    table = Texttable(max_width=800)
    table.set_deco(Texttable.HEADER)
    table.set_cols_align(['l', 'l', 'l'])
    all_apps = [['NAME', 'STATE', 'CREATED']]
    for app in res:
        count += 1
        date = parse_psql_date_str(app['timestamp'])
        all_apps.append(
            [app['name'], maintenance(app['maintenance']), reltime(date)]
        )

    table.add_rows(rows=all_apps)

    if count == 0:
        click.echo('No application found. Create your first app with')
        click.echo(click.style('$ story apps create', fg='magenta'))
    else:
        click.echo(table.draw())


def _is_git_repo_good():
    try:
        assert cli.run('git status 2&>1')
    except:
        click.echo(
            'Please create your application from a git-backed project folder.'
        )
        click.echo(click.style('$ git init', bold=True, fg='magenta'))
        sys.exit(1)

    try:
        # This will raise an error if a remote by the name of asyncy does
        # not exist.
        # remote = cli.run('git remote get-url asyncy')
        click.echo(
            click.style(
                'There appears to be git remote named asyncy '
                'already ({remote}).\n',
                fg='red',
            )
        )
        click.echo(
            'If you\'re trying to create a new app, please create a\n'
            'new directory with a git repository '
            'in there.'
        )
        sys.exit(1)
    except subprocess.CalledProcessError:
        # This just means that the remote does not exist, which is OK.
        pass


@apps.command()
@click.argument('name', nargs=1, required=False)
@click.option(
    '--team', type=str, help='Team name that owns this new Application'
)
def create(name, team):
    """Create a new app."""

    cli.user()
    story_yaml = cli.find_story_yml()

    if story_yaml is not None:
        click.echo(
            click.style(
                'There appears to be an Storyscript Cloud project in '
                f'{story_yaml} already.\n',
                fg='red',
            )
        )
        click.echo(
            click.style(
                'Are you trying to deploy? ' 'Try the following:', fg='red'
            )
        )
        click.echo(click.style('$ story deploy', fg='magenta'))
        sys.exit(1)

    # _is_git_repo_good()

    name = name or awesome.new()

    click.echo('Creating application… ', nl=False)

    with spinner():
        api.Apps.create(name=name, team=team)

    click.echo(
        '\b' + click.style(emoji.emojize(':heavy_check_mark:'), fg='green')
    )

    # click.echo('Adding git-remote... ', nl=False)
    # cli.run(f'git remote add asyncy https://git.asyncy.com/{name}')
    # click.echo(click.style(emoji.emojize(':heavy_check_mark:'), fg='green'))

    click.echo('Creating story.yml… ', nl=False)
    cli.settings_set(f'app_name: {name}\n', 'story.yml')

    click.echo(
        '\b' + click.style(emoji.emojize(':heavy_check_mark:'), fg='green')
    )

    click.echo('\nApp Name: ' + click.style(name, bold=True))
    click.echo(
        'App URL:  '
        + click.style(f'https://{name}.storyscriptapp.com/', fg='blue')
        + '\n'
    )

    click.echo(
        'You are now ready to write your first Storyscript! '
        f'{emoji.emojize(":party_popper:")}'
    )
    click.echo()

    cli.track('App Created', {'App name': name})

    click.echo(' - [ ] Write a Story:')
    click.echo(
        '       $ '
        + click.style('story bootstrap http > http.story', fg='magenta')
    )
    click.echo()
    click.echo(' - [ ] Deploy to Storyscript Cloud:')
    click.echo('       $ ' + click.style('story deploy', fg='magenta'))
    click.echo()
    click.echo('We hope you enjoy your deployment experience!')


@apps.command()
@options.app()
def url(app):
    """Display the full URL of an app.

    Great to use with $(story apps url) in bash.
    """
    cli.user()
    print_nl = False

    if os.isatty(sys.stdout.fileno()):
        print_nl = True

    click.echo(f'https://{app}.storyscriptapp.com/', nl=print_nl)


@apps.command()
@options.app()
@click.option(
    '--confirm', is_flag=True, help='Do not prompt to confirm destruction.'
)
def destroy(confirm, app):
    """Destroy an app."""

    cli.user()

    if confirm or click.confirm(
        f'Do you want to destroy {app!r}?', abort=True
    ):
        click.echo(f'Destroying application {app!r}… ', nl=False)

        with spinner():
            api.Apps.destroy(app=app)
            cli.track('App Destroyed', {'App name': app})

        click.echo(
            '\b' + click.style(emoji.emojize(':heavy_check_mark:'), fg='green')
        )



# -*- coding: utf-8 -*-
import sys

try:
    from .cli import cli
    from .commands import *
except KeyboardInterrupt:
    print('Aborted!')
    sys.exit(1)

if __name__ == '__main__':
    try:
        cli()
    except KeyboardInterrupt:
        print('Aborted!')
        sys.exit(1)



# -*- coding: utf-8 -*-
from random import choice, randint


left = (
    'admiring', 'adoring', 'affectionate', 'amazing', 'lekker',
    'awesome', 'blissful', 'brave', 'charming', 'clever',
    'cool', 'compassionate', 'competent', 'condescending',
    'confident', 'crazy', 'dazzling', 'determined', 'distracted',
    'dreamy', 'eager', 'ecstatic', 'elastic', 'elated', 'elegant', 'eloquent',
    'epic', 'fervent', 'festive', 'flamboyant', 'focused', 'friendly',
    'frosty', 'gallant', 'gifted', 'goofy', 'gracious', 'happy', 'hardcore',
    'heuristic', 'hopeful', 'hungry', 'infallible', 'inspiring', 'jolly',
    'jovial', 'keen', 'kind', 'laughing', 'loving', 'lucid', 'magical',
    'mystifying', 'modest', 'musing', 'naughty', 'nifty', 'nostalgic',
    'objective', 'optimistic', 'peaceful', 'pedantic', 'pensive', 'practical',
    'priceless', 'quirky', 'quizzical', 'recursing', 'relaxed', 'reverent',
    'romantic', 'serene', 'sharp', 'silly', 'sleepy',
    'stoic', 'stupefied', 'suspicious', 'sweet', 'tender', 'thirsty',
    'trusting', 'unruffled', 'upbeat', 'vibrant', 'vigilant', 'vigorous',
    'wizardly', 'wonderful', 'xenodochial', 'youthful', 'zealous', 'zen'
)

right = (
    'albattani', 'allen', 'almeida', 'antonelli', 'agnesi', 'archimedes',
    'ardinghelli', 'aryabhata', 'austin', 'babbage', 'banach', 'banzai',
    'bardeen', 'bartik', 'bassi', 'beaver', 'bell', 'benz', 'bhabha',
    'bhaskara', 'blackburn', 'blackwell', 'bohr', 'booth', 'borg', 'bose',
    'boyd', 'brahmagupta', 'brattain', 'brown', 'buck', 'burnell', 'cannon',
    'carson', 'cartwright', 'chandrasekhar', 'chaplygin', 'chatelet',
    'chatterjee', 'chebyshev', 'cohen', 'chaum', 'clarke',
    'colden', 'cori', 'cray', 'curran', 'curie', 'darwin', 'davinci',
    'dewdney', 'dhawan', 'diffie', 'dijkstra', 'dirac', 'driscoll', 'dubinsky',
    'easley', 'edison', 'einstein', 'elbakyan', 'elgamal', 'elion', 'ellis',
    'engelbart', 'euclid', 'euler', 'faraday', 'feistel', 'fermat',
    'fermi', 'feynman', 'franklin', 'gagarin', 'galileo', 'galois', 'ganguly',
    'gates', 'gauss', 'germain', 'goldberg', 'goldstine', 'goldwasser',
    'golick', 'goodall', 'greider', 'grothendieck', 'haibt', 'hamilton',
    'haslett', 'hawking', 'hellman', 'heisenberg', 'hermann', 'herschel',
    'hertz', 'heyrovsky', 'hodgkin', 'hofstadter', 'hoover', 'hopper', 'hugle',
    'hypatia', 'ishizaka', 'jackson', 'jang', 'jennings', 'jepsen', 'johnson',
    'joliot', 'jones', 'kalam', 'kapitsa', 'kare', 'keldysh', 'keller',
    'kepler', 'khayyam', 'khorana', 'kilby', 'kirch', 'knuth', 'kowalevski',
    'lalande', 'lamarr', 'lamport', 'leakey', 'leavitt', 'lederberg',
    'lehmann', 'lewin', 'lichterman', 'liskov', 'lovelace', 'lumiere',
    'mahavira', 'margulis', 'matsumoto', 'maxwell', 'mayer', 'mccarthy',
    'mcclintock', 'mclaren', 'mclean', 'mcnulty', 'mendel', 'mendeleev',
    'meitner', 'meninsky', 'merkle', 'mestorf', 'minsky', 'mirzakhani',
    'moore', 'morse', 'murdock', 'moser', 'napier', 'nash', 'neumann',
    'newton', 'nightingale', 'nobel', 'noether', 'northcutt', 'noyce',
    'panini', 'pare', 'pascal', 'pasteur', 'payne', 'perlman', 'pike',
    'poincare', 'poitras', 'proskuriakova', 'ptolemy', 'raman', 'ramanujan',
    'ride', 'montalcini', 'ritchie', 'rhodes', 'robinson', 'roentgen',
    'rosalind', 'rubin', 'saha', 'sammet', 'sanderson', 'shannon', 'shaw',
    'shirley', 'shockley', 'shtern', 'sinoussi', 'snyder', 'solomon', 'spence',
    'sutherland', 'stallman', 'stonebraker', 'swanson', 'swartz', 'swirles',
    'taussig', 'tereshkova', 'tesla', 'tharp', 'thompson', 'torvalds',
    'tu', 'turing', 'varahamihira', 'vaughan', 'visvesvaraya', 'volhard',
    'villani', 'wescoff', 'wiles', 'williams', 'williamson', 'wilson',
    'wing', 'wozniak', 'wright', 'wu', 'yalow', 'yonath', 'zhukovsky'
)


def new():
    return f'{choice(left)}-{choice(right)}-{randint(1000, 9999)}'



# -*- coding: utf-8 -*-

import json
import os
import subprocess
import sys
import time
import typing
from urllib.parse import urlencode
from uuid import uuid4

from blindspin import spinner

import click

import click_help_colors

import emoji


from raven import Client

from requests import Session

from .helpers.didyoumean import DYMGroup
from .version import version

# Initiate requests session, for connection pooling.
requests = Session()

# Typing hints.
Content = typing.Union[str, typing.Mapping, typing.List]

if not os.getenv('TOXENV'):
    enable_reporting = True
    sentry = Client(
        'https://007e7d135737487f97f5fe87d5d85b55@sentry.io/1206504')
else:
    enable_reporting = False
    sentry = Client()

data = None
home = os.path.expanduser('~/.storyscript')
old_home = os.path.expanduser('~/.asyncy')


def get_access_token():
    return data['access_token']


def get_user_id():
    return data['id']


def track_profile():
    _make_tracking_http_request(
        'https://stories.storyscriptapp.com/track/profile', {
            'id': str(get_user_id()),
            'profile': {
                'Name': data['name'],
                'Email': data.get('email'),
                'GitHub Username': data.get('username'),
                'Timezone': time.tzname[time.daylight],
            },
        },
    )


def _make_tracking_http_request(url, json_data):
    """
    Forks the HTTP call into the background.
    The caller does not make the call, and this method returns just after
    forking the process.

    The child makes the call, and does not return back to the caller,
    but instead just exits (sys.exit(0)). This is to ensure that whatever
    needs to happen after this method is called, does not happen twice.
    """
    if not enable_reporting:
        return
    if sys.platform != 'linux' and sys.platform != 'darwin':
        try:
            requests.post(url, json=json_data)
        except Exception:
            # ignore issues with tracking
            pass

        return

    pid = os.fork()
    if pid > 0:
        # This is the parent process.
        return

    try:
        requests.post(url, json=json_data)
    except Exception:
        # ignore issues with tracking
        pass

    # This is the child process. Exit now.
    sys.exit(0)


def track(event_name, extra: dict = None):
    if extra is None:
        extra = {}

    extra['CLI version'] = version
    _make_tracking_http_request(
        'https://stories.storyscriptapp.com/track/event', {
            'id': str(get_user_id()),
            'event_name': event_name,
            'event_props': extra
        })


def find_story_yml():
    """Finds './story.yml'."""
    path = _find_story_yml('story.yml')
    if not path:
        path = _find_story_yml('asyncy.yml')

    return path


def _find_story_yml(file_name):
    current_dir = os.getcwd()
    while True:
        if os.path.exists(f'{current_dir}{os.path.sep}{file_name}'):
            return f'{current_dir}{os.path.sep}{file_name}'
        elif current_dir == os.path.dirname(current_dir):
            break
        else:
            current_dir = os.path.dirname(current_dir)

    return None


def get_app_name_from_yml() -> str:
    file = find_story_yml()
    if file is None:
        return None
    import yaml

    with open(file, 'r') as s:
        return yaml.safe_load(s).pop('app_name')


def get_asyncy_yaml() -> dict:
    file = find_story_yml()
    # Anybody calling this must've already checked for this file presence.
    assert file is not None

    import yaml

    with open(file, 'r') as s:
        return yaml.safe_load(s)


def settings_set(content: Content, location: str):
    """Overwrites settings, to given location and content."""

    # Ensure the path is created and exists..
    loc_dir = os.path.abspath(location)
    loc_dir = os.path.dirname(loc_dir)

    os.makedirs(loc_dir, exist_ok=True)

    # If content is an object-like...
    if isinstance(content, (list, dict)):
        content = json.dumps(content, indent=2)

    # Write to the file.
    with open(location, 'w+') as file:
        file.write(content)


def initiate_login():
    global data

    click.echo(
        'Hi! Thank you for using ' + click.style('Storyscript Cloud',
                                                 fg='magenta') + '.'
    )
    click.echo('Please login with GitHub to get started.')

    state = uuid4()

    query = {'state': state}

    url = f'https://stories.storyscriptapp.com/github?{urlencode(query)}'

    click.launch(url)
    click.echo()
    click.echo("Visit this link if your browser doesn't open automatically:")
    click.echo(url)
    click.echo()

    while True:
        with spinner():
            try:
                url = 'https://stories.storyscriptapp.com' \
                      '/github/oauth_callback'
                r = requests.get(url=url, params={'state': state})

                if r.text == 'null':
                    raise IOError()

                r.raise_for_status()
                break

            except IOError:
                time.sleep(0.5)

            except KeyboardInterrupt:
                click.echo('Login failed. Please try again.')
                sys.exit(1)

    if r.json().get('beta') is False:
        click.echo('Hello! Storyscript Cloud is in private beta at this time.')
        click.echo(
            'We\'ve added you to our beta testers queue, '
            'and you should hear from us\nshortly via email'
            ' (which is linked to your GitHub account).'
        )
        sys.exit(1)

    settings_set(r.text, f'{home}/config')
    init()

    click.echo(emoji.emojize(':waving_hand:') + f'  Welcome {data["name"]}!')
    click.echo()
    click.echo('Create a new app with:')
    print_command('story apps create')

    click.echo()

    click.echo('To list all your apps:')
    print_command('story apps')

    click.echo()
    track('Login Completed')
    track_profile()


def user() -> dict:
    """Get the active user."""

    global data

    if data:
        return data
    else:
        initiate_login()
        return data


def print_command(command):
    """Prints a command to the CLI."""

    click.echo('  $ ' + click.style(command, fg='magenta'))


def assert_project(command, app, default_app, allow_option):
    if app is None:
        click.echo(click.style('No StoryScript Cloud application found.',
                               fg='red'))
        click.echo()
        click.echo('Create an application with:')

        print_command('story apps create')

        sys.exit(1)

    elif not allow_option and app != default_app:
        click.echo(
            click.style(
                'The --app option is not allowed with the {} command.'.format(
                    command),
                fg='red',
            )
        )
        sys.exit(1)
    return app


def init():
    global data

    config_file_path = f'{home}/config'
    old_config_file_path = f'{old_home}/.config'

    if os.path.exists(config_file_path):

        with open(config_file_path, 'r') as file:
            data = json.load(file)
            sentry.user_context({
                'id': get_user_id(),
                'email': data['email']
            })
    elif os.path.exists(old_config_file_path):

        with open(old_config_file_path, 'r') as old_config:
            data = json.load(old_config)
            settings_set(data, config_file_path)

        os.remove(old_config_file_path)
        init()


def stream(cmd: str):
    process = subprocess.Popen(cmd.split(' '), stdout=subprocess.PIPE)

    while True:

        output = process.stdout.readline()

        if output == b'' and process.poll() is not None:
            break

        if output:
            click.echo(output.strip())


def run(cmd: str):
    output = subprocess.run(
        cmd.split(' '), check=True, stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )
    return str(output.stdout.decode('utf-8').strip())


# def _colorize(text, color=None):
#     # PATCH for https://github.com/r-m-n/click-help-colors/pull/3
#     from click.termui import _ansi_colors, _ansi_reset_all
#     if not color:
#         return text
#     try:
#         return '\033[%dm' % (_ansi_colors[color]) + text + _ansi_reset_all
#     except ValueError:
#         raise TypeError('Unknown color %r' % color)
#
#
# click_help_colors._colorize = _colorize


class CLI(DYMGroup, click_help_colors.HelpColorsGroup):
    pass


@click.group(cls=CLI, help_headers_color='yellow',
             help_options_color='magenta')
def cli():
    """
    Hello! Welcome to Storyscript.

    We hope you enjoy, & we look forward to your feedback!

    Documentation: https://docs.storyscript.io/
    """
    init()
